.. title: ytree
.. slug: ytree
.. date: 2024-02-17 18:24:18 UTC-05:00
.. tags: 
.. category: 
.. link: 
.. description: 
.. type: text

Winter called for more children. This week's is named `ytree
<https://sr.ht/~yieldsfalsehood/ytree>`__! His heritage traces back
through some earlier projects, and this week I felt a compulsion to
give him new shape.

.. code-block:: shell

    ./foo type=dir
    ./foo/bar type=file content=foo/bar.yaml

Arch packaging uses ``MTREE`` as a programmable archiving format. I
built on the concept by switching from the bespoke line based format
to a YAML encoding with an archive entry per document. A complete
archive is serialized from a stream of independent documents.

.. include:: files/ytree/example.yaml
   :code: yaml

The file resource type produces the most familiar behavior, populating
archive contents from local files. I also added content providers that
allow for specifying object data inline, rendered from a template, or
fetched remotely.

.. code-block:: python

    def entries(version):
        return [
            ytree.tar.File(
                "version",
                ytree.res.String(version),
            ),
        ]

All of the mapped types have public python APIs so they can be
generated programmatically. A function like this could encapsulate a
version number on the fly. That could be transformed directly into a
tarchive or serialized to YAML and served elsewhere.

.. include:: files/version.yaml
   :code: yaml

I could build a sort of tree of archives with indirection. The first
file could represent some set of metrics each named by a path and this
second one a catalog with the particular "database" name given by the
path for dispatching.

.. include:: files/status.yaml
   :code: yaml

That evaluates to a tarchive with one entry populated by a snapshot of
that URL. I can access a specific configuration item in there by
deferring to ``tar`` to "extract" the database (``version.yaml``)
first then the item stored as a file (``version``) inside it.

.. include:: files/ytree/monitor.sh
   :code: sh

There's currently unoccupied space to configure SSL trust for those
types of resources. So far I do have runtime configurable support for
most jinja template loaders. Template resources themselves bind
context data to a template name, which are resolved at
runtime. There's no telling what this might look like when rendered!

.. include:: files/ytree/template.yaml
   :code: yaml

There is effectively no loader by default, which is fine if you never
reference a template. If you do then you need glue, like this ad hoc
loader that embeds a template body into the command line so you can
see the secret message.

.. include:: files/ytree/template.sh
   :code: sh
